@@ -5,6 +5,7 @@ my $r = Apache->request;
 
 $r->status(200);
 my $uri = $r->prev->uri;
+my $reason = $r->prev->subprocess_env("AuthCookieReason");
 
 my $form = <<HERE;
 <HTML>
@@ -18,7 +19,7 @@ my $form = <<HERE;
 <H1>This is a secure document</H1>
 </TD></TR>
 <TR><TD ALIGN=LEFT>
-<P>Please enter your login and password to authenticate.</P>
+<P>Failure reason: '$reason'.  Please enter your login and password to authenticate.</P>
 </TD>
 <TR><TD>
 <INPUT TYPE=hidden NAME=destination VALUE="$uri">
